@@ -1,4 +1,4 @@
-/* $OpenBSD: s_client.c,v 1.36 2018/02/11 20:03:10 jmc Exp $ */
+/* $OpenBSD: s_client.c,v 1.37 2018/11/14 06:24:21 tb Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -891,7 +891,7 @@ re_start:
 		BIO_free(fbio);
 		if (!foundit)
 			BIO_printf(bio_err,
-			    "didn't found STARTTLS in server response,"
+			    "didn't find STARTTLS in server response,"
 			    " try anyway...\n");
 		BIO_printf(sbio, ". STARTTLS\r\n");
 		BIO_read(sbio, sbuf, BUFSIZZ);
