@@ -1,4 +1,4 @@
-/* $OpenBSD: s_time.c,v 1.24 2018/07/13 18:36:56 cheloha Exp $ */
+/* $OpenBSD: s_time.c,v 1.32 2018/09/17 15:37:35 cheloha Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -90,7 +90,9 @@
 extern int verify_depth;
 
 static void s_time_usage(void);
-static SSL *doConnection(SSL * scon);
+static int run_test(SSL *);
+static int benchmark(int);
+static void print_tally_mark(SSL *);
 
 static SSL_CTX *tm_ctx = NULL;
 static const SSL_METHOD *s_time_meth = NULL;
@@ -225,18 +227,6 @@ s_time_usage(void)
 	options_usage(s_time_options);
 }
 
-/***********************************************************************
- * TIME - time functions
- */
-#define START	TM_RESET
-#define STOP	TM_GET
-
-static double
-tm_Time_F(int op)
-{
-	return app_timer_user(op);
-}
-
 /***********************************************************************
  * MAIN - main processing area for client
  *			real name depends on MONOLITH
@@ -244,13 +234,7 @@ tm_Time_F(int op)
 int
 s_time_main(int argc, char **argv)
 {
-	double totalTime = 0.0;
-	int nConn = 0;
-	SSL *scon = NULL;
-	time_t finishtime;
 	int ret = 1;
-	char buf[1024 * 8];
-	int ver;
 
 	if (single_execution) {
 		if (pledge("stdio rpath inet dns", NULL) == -1) {
@@ -259,7 +243,7 @@ s_time_main(int argc, char **argv)
 		}
 	}
 
-	s_time_meth = SSLv23_client_method();
+	s_time_meth = TLS_client_method();
 
 	verify_depth = 0;
 
@@ -321,160 +305,24 @@ s_time_main(int argc, char **argv)
 		/* goto end; */
 	}
 
-	if (!(s_time_config.perform & 1))
-		goto next;
-	printf("Collecting connection statistics for %lld seconds\n",
-	    (long long)s_time_config.maxtime);
-
 	/* Loop and time how long it takes to make connections */
-
-	bytes_read = 0;
-	finishtime = time(NULL) + s_time_config.maxtime;
-	tm_Time_F(START);
-	for (;;) {
-		if (finishtime < time(NULL))
-			break;
-		if ((scon = doConnection(NULL)) == NULL)
+	if (s_time_config.perform & 1) {
+		printf("Collecting connection statistics for %lld seconds\n",
+		    (long long)s_time_config.maxtime);
+		if (benchmark(0))
 			goto end;
-
-		if (s_time_config.www_path != NULL) {
-			int i, retval = snprintf(buf, sizeof buf,
-			    "GET %s HTTP/1.0\r\n\r\n", s_time_config.www_path);
-			if ((size_t)retval >= sizeof buf) {
-				fprintf(stderr, "URL too long\n");
-				goto end;
-			}
-			SSL_write(scon, buf, strlen(buf));
-			while ((i = SSL_read(scon, buf, sizeof(buf))) > 0)
-				bytes_read += i;
-		}
-		if (s_time_config.no_shutdown)
-			SSL_set_shutdown(scon, SSL_SENT_SHUTDOWN |
-			    SSL_RECEIVED_SHUTDOWN);
-		else
-			SSL_shutdown(scon);
-
-		nConn += 1;
-		if (SSL_session_reused(scon))
-			ver = 'r';
-		else {
-			ver = SSL_version(scon);
-			if (ver == TLS1_VERSION)
-				ver = 't';
-			else if (ver == SSL3_VERSION)
-				ver = '3';
-			else if (ver == SSL2_VERSION)
-				ver = '2';
-			else
-				ver = '*';
-		}
-		fputc(ver, stdout);
-		fflush(stdout);
-
-		SSL_free(scon);
-		scon = NULL;
 	}
-	totalTime += tm_Time_F(STOP);	/* Add the time for this iteration */
-
-	printf("\n\n%d connections in %.2fs; %.2f connections/user sec, bytes read %ld\n",
-	    nConn, totalTime, ((double) nConn / totalTime), bytes_read);
-	printf("%d connections in %lld real seconds, %ld bytes read per connection\n",
-	    nConn,
-	    (long long)(time(NULL) - finishtime + s_time_config.maxtime),
-	    bytes_read / nConn);
-
 	/*
 	 * Now loop and time connections using the same session id over and
 	 * over
 	 */
-
- next:
-	if (!(s_time_config.perform & 2))
-		goto end;
-	printf("\n\nNow timing with session id reuse.\n");
-
-	/* Get an SSL object so we can reuse the session id */
-	if ((scon = doConnection(NULL)) == NULL) {
-		fprintf(stderr, "Unable to get connection\n");
-		goto end;
-	}
-	if (s_time_config.www_path != NULL) {
-		int retval = snprintf(buf, sizeof buf,
-		    "GET %s HTTP/1.0\r\n\r\n", s_time_config.www_path);
-		if ((size_t)retval >= sizeof buf) {
-			fprintf(stderr, "URL too long\n");
+	if (s_time_config.perform & 2) {
+		printf("\n\nNow timing with session id reuse.\n");
+		if (benchmark(1))
 			goto end;
-		}
-		SSL_write(scon, buf, strlen(buf));
-		while (SSL_read(scon, buf, sizeof(buf)) > 0);
 	}
-	if (s_time_config.no_shutdown)
-		SSL_set_shutdown(scon, SSL_SENT_SHUTDOWN |
-		    SSL_RECEIVED_SHUTDOWN);
-	else
-		SSL_shutdown(scon);
-
-	nConn = 0;
-	totalTime = 0.0;
-
-	finishtime = time(NULL) + s_time_config.maxtime;
-
-	printf("starting\n");
-	bytes_read = 0;
-	tm_Time_F(START);
-
-	for (;;) {
-		if (finishtime < time(NULL))
-			break;
-		if ((doConnection(scon)) == NULL)
-			goto end;
-
-		if (s_time_config.www_path) {
-			int i, retval = snprintf(buf, sizeof buf,
-			    "GET %s HTTP/1.0\r\n\r\n", s_time_config.www_path);
-			if ((size_t)retval >= sizeof buf) {
-				fprintf(stderr, "URL too long\n");
-				goto end;
-			}
-			SSL_write(scon, buf, strlen(buf));
-			while ((i = SSL_read(scon, buf, sizeof(buf))) > 0)
-				bytes_read += i;
-		}
-		if (s_time_config.no_shutdown)
-			SSL_set_shutdown(scon, SSL_SENT_SHUTDOWN |
-			    SSL_RECEIVED_SHUTDOWN);
-		else
-			SSL_shutdown(scon);
-
-		nConn += 1;
-		if (SSL_session_reused(scon))
-			ver = 'r';
-		else {
-			ver = SSL_version(scon);
-			if (ver == TLS1_VERSION)
-				ver = 't';
-			else if (ver == SSL3_VERSION)
-				ver = '3';
-			else if (ver == SSL2_VERSION)
-				ver = '2';
-			else
-				ver = '*';
-		}
-		fputc(ver, stdout);
-		fflush(stdout);
-	}
-	totalTime += tm_Time_F(STOP);	/* Add the time for this iteration */
-
-	printf("\n\n%d connections in %.2fs; %.2f connections/user sec, bytes read %ld\n", nConn, totalTime, ((double) nConn / totalTime), bytes_read);
-	printf("%d connections in %lld real seconds, %ld bytes read per connection\n",
-	    nConn,
-	    (long long)(time(NULL) - finishtime + s_time_config.maxtime),
-	    bytes_read / nConn);
-
 	ret = 0;
  end:
-	SSL_free(scon);
-
 	if (tm_ctx != NULL) {
 		SSL_CTX_free(tm_ctx);
 		tm_ctx = NULL;
@@ -484,44 +332,32 @@ s_time_main(int argc, char **argv)
 }
 
 /***********************************************************************
- * doConnection - make a connection
+ * run_test - make a connection, get a file, and shut down the connection
+ *	
  * Args:
- *		scon	= earlier ssl connection for session id, or NULL
+ *		scon	= SSL connection
  * Returns:
- *		SSL *	= the connection pointer.
+ *		1 on success, 0 on error
  */
-static SSL *
-doConnection(SSL * scon)
+static int
+run_test(SSL *scon)
 {
+	char buf[1024 * 8];
 	struct pollfd pfd[1];
-	SSL *serverCon;
 	BIO *conn;
 	long verify_error;
-	int i;
+	int i, retval;
 
 	if ((conn = BIO_new(BIO_s_connect())) == NULL)
-		return (NULL);
-
-/*	BIO_set_conn_port(conn,port);*/
+		return 0;
 	BIO_set_conn_hostname(conn, s_time_config.host);
-
-	if (scon == NULL)
-		serverCon = SSL_new(tm_ctx);
-	else {
-		serverCon = scon;
-		SSL_set_connect_state(serverCon);
-	}
-
-	SSL_set_bio(serverCon, conn, conn);
-
-	/* ok, lets connect */
+	SSL_set_connect_state(scon);
+	SSL_set_bio(scon, conn, conn);
 	for (;;) {
-		i = SSL_connect(serverCon);
+		i = SSL_connect(scon);
 		if (BIO_sock_should_retry(i)) {
 			BIO_printf(bio_err, "DELAY\n");
-
-			i = SSL_get_fd(serverCon);
-			pfd[0].fd = i;
+			pfd[0].fd = SSL_get_fd(scon);
 			pfd[0].events = POLLIN;
 			poll(pfd, 1, -1);
 			continue;
@@ -530,15 +366,104 @@ doConnection(SSL * scon)
 	}
 	if (i <= 0) {
 		BIO_printf(bio_err, "ERROR\n");
-		verify_error = SSL_get_verify_result(serverCon);
+		verify_error = SSL_get_verify_result(scon);
 		if (verify_error != X509_V_OK)
 			BIO_printf(bio_err, "verify error:%s\n",
 			    X509_verify_cert_error_string(verify_error));
 		else
 			ERR_print_errors(bio_err);
-		if (scon == NULL)
-			SSL_free(serverCon);
-		return NULL;
+		return 0;
 	}
-	return serverCon;
+	if (s_time_config.www_path != NULL) {
+		retval = snprintf(buf, sizeof buf,
+		    "GET %s HTTP/1.0\r\n\r\n", s_time_config.www_path);
+		if (retval == -1 || retval >= sizeof buf) {
+			fprintf(stderr, "URL too long\n");
+			return 0;
+		}
+		if (SSL_write(scon, buf, retval) != retval)
+			return 0;
+		while ((i = SSL_read(scon, buf, sizeof(buf))) > 0)
+			bytes_read += i;
+	}
+	if (s_time_config.no_shutdown)
+		SSL_set_shutdown(scon, SSL_SENT_SHUTDOWN |
+		    SSL_RECEIVED_SHUTDOWN);
+	else
+		SSL_shutdown(scon);
+	return 1;
+}
+
+static void
+print_tally_mark(SSL *scon)
+{
+	int ver;
+
+	if (SSL_session_reused(scon))
+		ver = 'r';
+	else {
+		ver = SSL_version(scon);
+		if (ver == TLS1_VERSION)
+			ver = 't';
+		else
+			ver = '*';
+	}
+	fputc(ver, stdout);
+	fflush(stdout);
+}
+
+static int
+benchmark(int reuse_session)
+{
+	double elapsed, totalTime;
+	int nConn = 0;
+	SSL *scon = NULL;
+	int ret = 1;
+
+	if (reuse_session) {
+		/* Get an SSL object so we can reuse the session id */
+		if ((scon = SSL_new(tm_ctx)) == NULL)
+			goto end;
+		if (!run_test(scon)) {
+			fprintf(stderr, "Unable to get connection\n");
+			goto end;
+		}
+		printf("starting\n");
+	}
+
+	nConn = 0;
+	bytes_read = 0;
+
+	app_timer_real(TM_RESET);
+	app_timer_user(TM_RESET);
+	for (;;) {
+		elapsed = app_timer_real(TM_GET);
+		if (elapsed > s_time_config.maxtime)
+			break;
+		if (scon == NULL) {
+			if ((scon = SSL_new(tm_ctx)) == NULL)
+				goto end;
+		}
+		if (!run_test(scon))
+			goto end;
+		nConn += 1;
+		print_tally_mark(scon);
+		if (!reuse_session) {
+			SSL_free(scon);
+			scon = NULL;
+		}
+	}
+	totalTime = app_timer_user(TM_GET);
+
+	printf("\n\n%d connections in %.2fs; %.2f connections/user sec, bytes read %ld\n",
+	    nConn, totalTime, ((double) nConn / totalTime), bytes_read);
+	printf("%d connections in %.0f real seconds, %ld bytes read per connection\n",
+	    nConn,
+	    elapsed,
+	    bytes_read / nConn);
+
+	ret = 0;
+ end:
+	SSL_free(scon);
+	return ret;
 }
